<?php defined('SYSPATH') or die('No direct script access.');

/**
 * Class Model_Teacher
 *
 * @property $ID                  int
 * @property $ExternalID          int
 * @property $LastName            string
 * @property $FirstName           string
 * @property $SecondName          string
 * @property $AccountID           int
 * @property $TeacherID           int
 * @property $JobPositionID       int
 * @property $JobPositionName     string
 * @property $DepID               int
 * @property $DepName             string
 * @property $FacultyID           int
 * @property $FacultyName         string
 * @property $FacultyAbbr         string
 * @property $Status              bool      Is the teacher working now or not?
 * @property-read $ActivationCode string  #todo: add field
 */
class Model_Teacher extends Model_Container
{
    // see `job_positions` table
    const JOB_TEACHER = 9;
    const JOB_PROFESSOR = 10;
    const JOB_SENIOR_TEACHER = 11;


    protected function getRawData($id) {
        $sql = 'CALL `Teacher_GetInfo`(:id)';
        $info = DB::query(Database::SELECT, $sql)
            ->param(':id', $id)->execute();

        if ($info->count() == 0)
            throw new InvalidArgumentException('Teacher with not found');
        return $info[0];
    }

    public static function make() {
        return new Model_Helper_TeacherBuilder();
    }

    # todo: conflict with self::load()
    public static function with($id) {
        return self::load($id);
    }

    public static function withExternalID($extid) {
        $sql = 'SELECT `Teacher_GetIDFromExternalID`(:ExtID) AS `ID`';
        $res = DB::query(Database::SELECT, $sql)
            ->param(':ExtID', $extid)
            ->execute()->get('ID');
        return $res;
    }

    protected function create() {
        if ( isset($this->data['ExternalID']) ) {
            $sql = 'SELECT `Teacher_CreateActivated`(LastName, FirstName, SecondName, JobPositionID, DepID, ExternalID) AS `ID`';

            $this->data[self::$ID_FIELD] = DB::query(Database::SELECT, $sql)
                ->parameters($this->data)
                ->execute()->get('ID');
        } else {
            $sql = 'SELECT `Teacher_Create`(LastName, FirstName, SecondName, JobPositionID, DepID, ActivationCode) AS `ID`';

            $this->data[self::$ID_FIELD] = DB::query(Database::SELECT, $sql)
                ->parameters($this->data)
                ->execute()->get('ID');
        }

        if ($this->ID <= 0)
            throw new InvalidArgumentException(Error::INVALID_PARAMETERS);
    }


    public function changeInfo($lastName, $firstName, $secondName, $degreeID, $departmentID, $isWorking=true) {
        $sql = 'SELECT `ChangeTeacherInfo`(:id, :last, :first, :second, :degree, :department, :status) AS `Num`';
        $res = DB::query(Database::SELECT, $sql)
            ->parameters([
                ':id'         => $this->ID,
                ':last'       => $lastName,
                ':first'      => $firstName,
                ':second'     => $secondName,
                ':degree'     => $degreeID,
                ':department' => $departmentID,
                ':status'     => $isWorking,
            ])->execute()->get('Num');
        return ($res == 0);
    }

    /**
     * @param int $semesterID
     * @return Model_Discipline[]
     */
    public function getDisciplines($semesterID) {
        $sql = 'CALL `GetDisciplinesForTeacher`(:id, :semesterID)';
        $query = DB::query(Database::SELECT, $sql)
            ->param(':semesterID', $semesterID)
            ->param(':id', $this->ID)
            ->execute();

        $list = [];
        foreach ($query as $data)
            $list[] = new Model_Discipline($data, true);

        return $list;
    }

    /**
     * Get edit rights for discipline.
     * @param $discipline Model_Discipline
     * @return int  1 if teacher is bound to discipline,
     *                    0 if he is from a dean office,
     *                    -1 otherwise.
     */
    public function getRightsForDiscipline(Model_Discipline $discipline) {
        $sql = 'SELECT `GetEditRightsForTeacher`(:teacher, :discipline) AS `Num`';
        return DB::query(Database::SELECT, $sql)
            ->param(':discipline', $discipline->ID)
            ->param(':teacher', $this->ID)
            ->execute()->get('Num');
    }
}
